
// sg - A scene graph representation.

// Copyright (C) 2005 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

// simple example just to exercise the x3d importer.

#ifdef HAVE_CONFIG_H
#include <openlibraries_global_config.hpp>
#endif

#include <openpluginlib/pl/openpluginlib.hpp>
#include <openpluginlib/pl/utf8_utils.hpp>

#include <openobjectlib/sg/openobjectlib_plugin.hpp>

using namespace std;

namespace opl = olib::openpluginlib;
namespace fs  = boost::filesystem;
namespace sg  = olib::openobjectlib::sg;

namespace
{
	const std::string default_filename = "test.x3d";
	
	struct query_traits
	{
		query_traits( const opl::wstring& filename )
			: filename_( filename )
		{ }
		
		opl::wstring libname( ) const
		{ return L"openobjectlib"; }
		
		opl::wstring to_match( ) const
		{ return filename_; }

		opl::wstring type( ) const
		{ return L""; }
		
		int merit( ) const
		{ return 0; }
		
		opl::wstring filename_;
	};
}

static sg::scene_ptr load_scene( const fs::path& path )
{
	typedef opl::discovery<query_traits> ool_discovery;
	
	ool_discovery plugins( query_traits( opl::to_wstring( path.native_file_string( ).c_str( ) ) ) );
	
	ool_discovery::const_iterator i = plugins.begin( );
	boost::shared_ptr<sg::openobjectlib_plugin> plug = boost::shared_static_cast<sg::openobjectlib_plugin>( i->create_plugin( "" ) );
	
	return plug->load( path );
}

int main( int argc, char* argv[ ] )
{
	opl::init( "./plugins" );

	std::string filename( default_filename );
	
	if( argc > 1 )
		filename = argv[ 1 ];
	
	load_scene( default_filename );
}
